# -*- encoding: utf-8 -*-
##############################################################################
#
#    Addition to OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2012 Vertel. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
import pooler
import time
import datetime


class fms_odometer(osv.osv):
    _name = "fms.odometer"
    _description = "Recording of daily odometer reading"
    
    
    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        
        for o in self.browse(cr, uid, ids, context=context):
            res[o.id] = {
                'podometer': o.vehicle_id.podometer,
                'distance': 0,
                'fuel': 0,
                'cost': 0,
            }
            for prev_o in self.pool.get('fms.odometer').browse(cr, uid,self.pool.get('fms.odometer').search(cr, uid, [('vehicle_id','=',o.vehicle_id.id)])):
                if o.odometer > 0 and o.odometer > prev_o.odometer:
                    if res[o.id]['podometer'] < prev_o.odometer:
                        res[o.id]['podometer'] = prev_o.odometer
                elif o.odometer == 0:
                    if res[o.id]['podometer'] < prev_o.odometer:
                        res[o.id]['podometer'] = prev_o.odometer

            res[o.id]['distance'] = o.odometer - res[o.id]['podometer'] 
            res[o.id]['fuel']     = o.vehicle_id.litrepm * res[o.id]['distance'] / 10
            res[o.id]['cost']     = o.vehicle_id.costpm * res[o.id]['distance'] 
        return res


    
    
    _columns = {
                'vehicle_id': fields.many2one('fms.vehicles', 'Vehicle', required=True, ondelete='cascade', select=True,),
                'date':fields.date('Date',required=True),
                'odometer':fields.integer('To',required=True),
                'podometer':fields.function(_amount_all,string='From',type = 'integer',help="previous recorded odometer",multi="podometer"),
                'distance':fields.function(_amount_all,string='Distance',type = 'integer',help="Distance entered during trip",multi="distance"),
                'fuel':fields.function(_amount_all,string='Fuel',type = 'integer',help="Fuel used during trip using vehicle l/100",multi="fuel"),
                'cost':fields.function(_amount_all,string='Cost',type = 'integer',help="Cost for the trip using vehicle cost per mile",multi="cost"),
                'note':fields.text('Note'),                
                'type':fields.selection([('official','Official Matter'),('private','Private'),],string='Type',readonly=False)
                }
    _defaults = {
        'date' : lambda *a: time.strftime('%Y-%m-%d'),
        'type': 'official',
    }


fms_odometer()


class fms_vehicles(osv.osv):
    _name = "fms.vehicles"
    _inherit = "fms.vehicles"


    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        self._lodometer.add('_lodometer_odometer')
        res = super(fms_vehicles,self)._amount_all(cr,uid,ids, field_name,arg,context)
        return res

    def last_lodometer(self,cr,uid,vehicle,context=None):
        res = super(fms_vehicles,self).last_lodometer(cr,uid,vehicle,context)
        lastom_id = self.pool.get('fms.odometer').search(cr, uid, [('odometer','>',res['lodometer']),('vehicle_id','=',vehicle.id)])
        if len(lastom_id) > 0:
            for lastom in self.pool.get('fms.odometer').browse(cr, uid, lastom_id, context=context):
                if lastom.date > res['bdate']:
                    res['bdate'] = lastom.date
                if lastom.odometer > res['lodometer']:
#                        raise osv.except_osv(_('Invalid action !'), _('vals! %s ' % lastom.odometer,))
                    res['lodometer'] = int(lastom.odometer)
        return res

    _columns = {
               'odometer': fields.one2many('fms.odometer', 'vehicle_id', 'Odometer recordings', readonly=True, states={'draft': [('readonly', False)]}),
                }
    
fms_vehicles()



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
