# -*- encoding: utf-8 -*-
##############################################################################
#
#    Addition to OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2012 Vertel. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from datetime import date, datetime, timedelta
from dateutil.relativedelta import relativedelta
import time
import pooler
from osv import fields, osv
from tools.translate import _
from tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, float_compare
import decimal_precision as dp
import netsvc
from tools.translate import _    
import calendar


class fms_service_items(osv.osv):
    _name = "fms.service.items"
    _description="Different types of service tasks"
    
    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        res = {}        
        for item in self.browse(cr, uid, ids, context=context):
            res[item.id] = {
                'freq_days' : item.freqn,
                'freq_miles': item.freqd,
                'costpm': 0.0,
            }
            
            if item.freqn > 0 and item.freqz == 'weeks':
                res[item.id]['freq_days'] = item.freqn * 7
            if item.freqn > 0 and item.freqz == 'months':
                res[item.id]['freq_days'] = item.freqn * 30.5
            if item.freqn > 0 and item.freqz == 'years':
                res[item.id]['freq_days'] = item.freqn * 365

            if item.freqdz == 'km':
                res[item.id]['freq_miles'] = item.freqd / 10

            if res[item.id]['freq_miles'] > 0:
                res[item.id]['costpm'] = round(item.cost / res[item.id]['freq_miles'],2)
                                
        return res
    
    _columns={
              'name':fields.char('Service Item',size=30,required=True),
              'itemtype':fields.selection([
                                         ('normal','Normal'),
                                         ('fluids','Fluids'),
                                         ('inspection','Inspection'),
                                         ('replace','Replace'),
                                         ('others','Others')],'Type of Service'),
              'freqn':fields.integer('Frequency'),
              'freqz':fields.selection([('days','Days'),
                                        ('weeks','Weeks'),
                                        ('months','Months'),
                                        ('years','Years')
                                        ]),
              'freq_days':fields.function(_amount_all,string='Frequency in days',type='integer',multi="freq_days"),
              'freq_miles':fields.function(_amount_all,string='Frequency in miles',type='integer',multi="freq_miles"),

              'freqd':fields.float('Mileage', digits=(10,0)),
              'freqdz':fields.selection([
                                         ('miles','Miles'),
                                         ('km','Kilometers'),
                                         ('hours','Hours')
                                         ]),
               'fixed_month':fields.selection([(1,_('January')),(2,_('February')),(3,_('Mars')),(4,_('April')),(5,_('May')),(6,_('June')),(7,_('July')),(8,_('August')),(9,_('September')),(10,_('October')),(11,_('November')),(12,_('December'))],
                                            'Fixed Month',help="Service Item will be performed at this time every year"),
               'sparesreq':fields.one2many('fms.service.items.spares','serviceitem','Spares Required'),
               'delay': fields.float('Delay',help="Time that the vehicle are out of service in Hours"),
               'wo':fields.one2many('fms.wo','servitem','Work Order Tasks'),
               'cost':fields.float('Cost',help="Approximate cost for service item"),
               'costpm':fields.function(_amount_all,string='Cost per Mile',type = 'float',help="Approximate cost per mile for all service items",multi="costpm"),
 
}
    _defaults = {
                 'cost': 0,
                 'freqdz':lambda *a:'km',
                 'freqz':lambda *a:'days',
                 'itemtype':lambda *a:'inspection',
                 'delay': 4.0,
                 }
fms_service_items()

class fms_service_items_spares(osv.osv):
    _name="fms.service.items.spares"
    _description="Service items to spare relation"
    
    def fetchunit(self,cr,uid,ids,productid):
        #print "Control is txed & Product id"
        #print productid
        if productid:
            product_obj = self.pool.get('product.product')
            product_obj = product_obj.browse(cr, uid, productid,False)
            #print product_obj
            default_uom = product_obj.uom_id and product_obj.uom_id.id
            #print default_uom
            res={}
            res['unit']=default_uom
            #print res
            return {'value':res}
        else:
            return False
    
    _columns={
              'product':fields.many2one('product.product','Spare Part',domain="[('spare_ok','=','True')]",required=True),
              'qtyreqd':fields.float('Quantity',digits=(10,3),required=True),
              #'unit':fields.function(defaultuom,method=True,store=True,string="Unit of Use(Default)"),
              'unit':fields.many2one('product.uom','Unit Of Use'),
              'serviceitem':fields.many2one('fms.service.items','Service Item'),
              }
fms_service_items_spares()

class fms_service_templ(osv.osv):
    _name = 'fms.service.templ'
    _description='The Template is a collection of service items.'
    
    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for o in self.browse(cr, uid, ids, context=context):
            res[o.id] = {
                'costpm': 0.0,
            }
            totalcost = 0.0
            for item in o.items:
                self.log(cr,uid,o.id,"Item %s %s" % (item.name,item.costpm))
                if item.freq_miles > 0:
                    totalcost += (item.costpm / item.freq_miles)
            res[o.id]['costpm'] = round(totalcost,2)

        return res

    def create_wo(self,cr,uid,ids,vehicle_id,context=None):
        today = date.today()   
        vehicle = self.pool.get('fms.vehicles').browse(cr,uid,vehicle_id)[0]
        if vehicle.bdate == 0:
            raise osv.except_osv(_('Invalid action !'), _('Please register statements in Odometer log or Fuel Log for the vehicle %s !') % vehicle.name,)
        if vehicle.pdate == 0:
            raise osv.except_osv(_('Invalid action !'), _('Please register purchase date for the vehicle %s !') % vehicle.name,)
        if vehicle.podometer == 0:
            raise osv.except_osv(_('Invalid action !'), _('Please register initial odometer statement at purchase for the vehicle %s !') % vehicle.name,)
        if vehicle.lodometer == 0:
            raise osv.except_osv(_('Invalid action !'), _('There are not enought data for the operation (Lodometer) for the vehicle %s !') % vehicle.name,)
        if vehicle.lmiles == 0:
            raise osv.except_osv(_('Invalid action !'), _('There are not enought data for the operation (Miles per Year missing) for the vehicle %s !') % vehicle.name,)

        items_todo = {}
        for item in self.browse(cr,uid,ids[0]).items:
            i = 0
            idate = datetime.strptime(vehicle.pdate,DEFAULT_SERVER_DATE_FORMAT)
            odometer = vehicle.podometer
            while i < 4:  # Until WO are created for this Service Item a year a head (or max 4)
                i += 1
                for wo in item.wo:
                    if wo.vehicle.id == vehicle.id:
                        if wo.odometer > odometer:
                            odometer = wo.odometer
                        if wo.scheddate > date:
                            idate = datetime.strptime(wo.scheddate,DEFAULT_SERVER_DATE_FORMAT)
                        elif wo.duedate > date:
                            idate = datetime.strptime(wo.duedate,DEFAULT_SERVER_DATE_FORMAT)
                    
                next_scheddate = next_duedate = ''
                #   In days, check if its room for another in the timeframe
                if item.freq_days > 0 and (idate + timedelta(days=item.freq_days)) < (datetime.today() + timedelta(days=365)):
                    idate += timedelta(days=item.freq_days)
                    if idate < datetime.today():
                        idate = datetime.today()
                    next_duedate = (idate).strftime(DEFAULT_SERVER_DATE_FORMAT)
                    next_scheddate = (idate - timedelta(days=item.freq_days*0.1)).strftime(DEFAULT_SERVER_DATE_FORMAT)                    
                #   In Miles, check if its room in the yearly distance and refrace distance to a date
                elif item.freq_miles > 0 and (odometer + item.freq_miles) < (vehicle.lodometer + vehicle.lmiles):
                    odometer += item.freq_miles
                    idate += timedelta(days=item.freq_miles / vehicle.lmiles * 365.0)
                    if idate < datetime.today():
                        idate = datetime.today() + timedelta(days=item.freq_miles / vehicle.lmiles * 365.0 * 0.1)
                    next_date = (idate).strftime(DEFAULT_SERVER_DATE_FORMAT)
                    next_scheddate = (idate - timedelta(days=item.freq_miles / vehicle.lmiles * 365.0 * 0.1)).strftime(DEFAULT_SERVER_DATE_FORMAT)
                #   Fixed month
                elif item.fixed_month > 0 and idate < datetime(datetime.today().year,item.fixed_month,calendar.monthrange(datetime.today().year,item.fixed_month)[1]):
                    idate = datetime(datetime.today().year,item.fixed_month,calendar.monthrange(datetime.today().year,item.fixed_month)[1])
                    if idate < datetime.today():
                        idate = datetime(datetime.today().year+1,item.fixed_month,calendar.monthrange(datetime.today().year,item.fixed_month)[1])
                    next_date = (idate).strftime(DEFAULT_SERVER_DATE_FORMAT)
                    next_scheddate = (idate - timedelta(days=15)).strftime(DEFAULT_SERVER_DATE_FORMAT)                    
                
                if next_duedate > '':
                    self.pool.get('fms.wo').create(cr,uid,{
                        'vehicle': vehicle.id,
                        'servitem': item.id,
                        'duedate': next_duedate,
                        'scheddate': next_scheddate,
                        'delay': item.delay,
                    },context)
                else:
                    break
#                raise osv.except_osv(_('Invalid action !'), _('Cannot change service item with spares not in draft ! %s | %s %s') % (next_date,item,vehicle),)

        
    _columns={
              'name':fields.char('Template Name',size=50,required=True,select=1,help="Service Templates are a collection of Service Tasks for a specific category of vehicles."),
              'description':fields.char('Description',size=50,required=True,select=1),
              'items':fields.many2many('fms.service.items','fms_template_tasks_rel','template_id','item_id','Service Items'),
              'vehicle_ids':fields.one2many('fms.vehicles','schedname','Vehicles'),
              'costpm':fields.function(_amount_all,string='Cost per Mile',type = 'float',help="Approximate cost per mile for all service items",multi="costpm"),
              }
fms_service_templ()




class fms_wo(osv.osv):
    _name="fms.wo"  # fms.wo.taks
    _description="Work Order Tasks"
    _table="fms_wo"
 
    def _add_spares(self, cr, uid, ids, servitem_id, context=None):
#        servitem = self.pool.get('fms.service.items.spares').browse(cr,uid,servitem_id)
        for spare in self.pool.get('fms.service.items').browse(cr,uid,servitem_id).sparesreq:
#            raise osv.except_osv(_('Invalid action !'), _('Cannot change service item with spares not in draft ! %s | %s') % (spare.product,spare.qtyreqd),)
            self.pool.get('fms.wo.tasks.spares').create(cr,uid,{
                    'wo'  : ids[0],
                    'product' : spare.product.id, 
                    'quantity': spare.qtyreqd,
                    'unit': spare.unit.id,
                    },context=None)

    
    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        res = {}        
        for wo in self.browse(cr, uid, ids, context=context):
            res[wo.id] = {
                'last_wo_days' : item.freqn,
                'last_wo_miles': item.freqd,   # Miles or Hours
            }
            lastwo_id = self.pool.get('fms.wo').search(cr, uid, [('vehicle_id','=',vehicle.id)])
            if len(lastwo_id) > 0:
                for lastwo in self.pool.get('fms.wo').browse(cr, uid, lastwo_id, context=context):
                    if lastwo.date > res[wo.id]['bdate']:
                        res[vehicle.id]['bdate'] = lastfl.date
                    if res[vehicle.id]['lodometer'] < lastfl.odometer:
                        res[vehicle.id]['lodometer'] = lastfl.odometer
                    
        return res

        

    _columns={
              'wo':fields.char('Work Order',size=20,required=True),
              'vehicle':fields.many2one('fms.vehicles','Vehicle',required=True),
              'servitem':fields.many2one('fms.service.items','Service Task',required=True),
              'odometer':fields.integer('Odometer',help="Odometer recording at service"),
              'duedate':fields.date('Due Date'),
              'scheddate':fields.date('Scheduled Date'),
              'delay':fields.float('Delay',help="Delay time to fullfill the service (time where the vehicle are out of service)"),              
              'wosysgen':fields.boolean('System Generated'),
              'spares':fields.one2many('fms.wo.tasks.spares','wo','Spares Required'),
              'state':fields.selection([ ('draft','Draft'),
                                         ('inprogress','In Progress'),
                                         ('complete','Complete'),
                                         ],'Task Status',required=True),              
              }
    _defaults={
               'wo':lambda obj,cr,uid,context:  obj.pool.get('ir.sequence').get(cr, uid, 'fms.wo',context),
               'state':lambda *a:'draft',
               'delay': 4.0,
               }
    
#    _sql_constraints = [
#        ('uniq_wo', 'unique (wo)', 'The task no must be unique !')
#    ]
    
    def on_change_servitem(self, cr, uid, ids, servitem_id, context=None):
#        raise osv.except_osv(_('Invalid action !'), _('Cannot change service item with spares not in draft ! %s | %s') % (ids,context),)
        spares = self.pool.get('fms.wo.tasks.spares').browse(cr,uid,self.pool.get('fms.wo.tasks.spares').search(cr,uid,[('wo','=',ids[0])]))
        for spare in spares:
            if spare.state == 'draft':
                self.pool.get('fms.wo.tasks.spares').unlink(cr,uid,spare.id)
            else:
                raise osv.except_osv(_('Invalid action !'), _('Cannot change service item with spares not in draft !'),)
        self._add_spares(cr,uid,ids,servitem_id,context)
        
    
    def button_inprogress(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'inprogress'})
    def button_draft(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'draft'})
    def button_complete(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'complete'})
        

fms_wo()

class fms_wo_tasks_spares(osv.osv):
    _name="fms.wo.tasks.spares"
    _description="WO Tasks spares requirement"
    _rec_name="spare_req_no"
    
    def wts_seq_get(self, cr, uid):
        return self.pool.get('ir.sequence').get(cr, uid, 'fms.wo.tasks.spares')
    
    def fetchunit(self,cr,uid,ids,productid):
        #print "Control is txed & Product id"
        #print productid
        if productid:
            product_obj = self.pool.get('product.product')
            product_obj = product_obj.browse(cr, uid, productid,False)
            #print product_obj
            default_uom = product_obj.uom_id and product_obj.uom_id.id
            #print default_uom
            res={}
            res['unit']=default_uom
            #print res
            return {'value':res}
        else:
            return False
    
    def create(self, cr, user, vals, context=None):
        name=self.pool.get('ir.sequence').get(cr, user, 'fms.wo.tasks.spares')
        return super(fms_wo_tasks_spares,self).create(cr, user, vals, context) 
        
    _columns={
             'spare_req_no':fields.char('Spares Req. No',size=20,required=True),
             'wo':fields.many2one('fms.wo','Parent WO Task',ondelete="cascade"),
             'product':fields.many2one('product.product',domain=[('spare_ok','=','True')],string='Spare Part',required=True),
             'quantity':fields.float('Quantity',required=True),
             'unit':fields.many2one('product.uom','Unit Of Use',required=True),
             'state':fields.selection([
                                       ('draft','Request Draft'),
                                       ('approved','Pending Move'),
                                       ('moved','Spare Issued')
                                       ],'State/Status',)
             }
    _defaults={
              'quantity': lambda *a: 1.00,
              'state': lambda *a:'draft',
              'spare_req_no': lambda obj,cr,uid,context: obj.pool.get('fms.wo.tasks.spares').wts_seq_get(cr,uid),
              }
    _sql_constraints = [
        ('spare_req_no', 'unique (spare_req_no)', 'The spare request no must be unique!')
    ]
fms_wo_tasks_spares()

class fms_wo_tasks_labour(osv.osv):
    _name="fms.wo.tasks.labour"
    _description="WO labour Tasks"
    _rec_name="lab_req_no"
    _columns={
             'lab_req_no':fields.char('Labour Req No',size=20,required=True),
             'partner':fields.many2one('res.partner','Workshop/Payee'),
             'description':fields.char('Description of work',size=64,required=True),
             'amount':fields.float('Amount',digits=(10,2),required=True),
             'state': fields.selection([
                                        ('draft','Draft'),
                                        ('verified','Verified'), 
                                        ('paid','Paid'),
                                        ('audited','Audited')
                                        ], 'Status', readonly=True),
             }
    _defaults={
              'amount': lambda *a: 1.00,
              'state':lambda *a: 'draft',
              }
fms_wo_tasks_labour()
    



class fms_vehicles(osv.osv):
    _name = "fms.vehicles"
    _inherit = "fms.vehicles"

    def _nextwo(self, cr, uid, ids, field_name, arg, context=None):
        res = {}        
        for vehicle in self.browse(cr, uid, ids, context=context):
            res[vehicle.id] = {
                'nextwo': '',                
            }

            (wo) = self.pool.get('fms.wo').browse(cr, uid, self.pool.get('fms.wo').search(cr, uid,[('vehicle','=',vehicle.id),('state','=','inprogress')],order="rdate desc", limit=1,context=context))
            if len(wo)>0:
                res[vehicle.id]['nextwo'] = wo[0].wo
            
        return res



    _columns = {
               'schedname':fields.many2one('fms.service.templ','PM Schedule',help="Preventive maintainance schedule for this vehicle",required=False),
               'work_order': fields.one2many('fms.wo', 'vehicle', 'Work Orders', readonly=True, states={'draft': [('readonly', False)]}),
               'nextwo':fields.function(_nextwo,string='Next WO',type = 'char',help="Next planned Work Order. If WO is missing the vehicle are possible out of WO-plans",multi="nextwo"),
                }

    def button_create_wo(self, cr, uid, ids, context=None):
        if not self.browse(cr,uid,ids[0]).schedname:
            raise osv.except_osv(_('Invalid action !'), _('Cannot create work orders without a template, assign one first !'),)
#        self.log(cr, uid, ids[0], "Here I come %s" % self.browse(cr,uid,ids[0]).name)
        return self.browse(cr,uid,ids[0]).schedname.create_wo(ids)

fms_vehicles()


