# -*- encoding: utf-8 -*-
##############################################################################
#
#    Addition to OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2012 Vertel. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
import pooler
import time
import datetime
from tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, float_compare
from tools.translate import _

class fms_vehicles(osv.osv):
    _name = "fms.vehicles"
    _inherits = {'account.analytic.account': "analytic_account_id"}

    _description = "Holds records of Vehicles"
    
    def last_lodometer(self,cr,uid,vehicle,context=None):
        return {
                'bdate': vehicle.pdate or 0,
                'lodometer': vehicle.podometer,
        }
    
    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
#        raise osv.except_osv(_('Invalid action !'), _('vals! %s ') % self._lodometer,)
        res = {}        
        for vehicle in self.browse(cr, uid, ids, context=context):
            res[vehicle.id] = self.last_lodometer(cr,uid,vehicle)
        return res

    def _amount_vehicle(self, cr, uid, ids, field_name, arg, context=None):
        res = {}
        for vehicle in self.browse(cr, uid, ids, context=context):
            res[vehicle.id] = {
                'lmiles': 0.0,
                'days': 0,
                'distance': 0.0,
                'costpm': 0.0,
                'roi': 0.0,
                'costpmon': 0.0,
            }
            #raise osv.except_osv(_('Invalid action !'), _('vals! %s | %s') % (vehicle.bdate or '',vehicle.pdate or ''),)
            res[vehicle.id]['days']     = (datetime.datetime.strptime(vehicle.bdate or '1900-1-1',DEFAULT_SERVER_DATE_FORMAT)-datetime.datetime.strptime(vehicle.pdate or '1900-1-1',DEFAULT_SERVER_DATE_FORMAT)).days
            res[vehicle.id]['distance'] = vehicle.lodometer - vehicle.podometer
            if res[vehicle.id]['distance'] > 0 and res[vehicle.id]['days'] > 0:
                res[vehicle.id]['lmiles'] = res[vehicle.id]['distance'] / res[vehicle.id]['days'] * 365
#            self.log(cr,uid,vehicle.id,"Vehicle %s lodometer %s podometer %s" % (vehicle.name,vehicle.lodometer,vehicle.podometer))
        return res


    def name_get(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if not len(ids):
            return []
        result = []
        for vehicle in self.browse(cr, uid, ids, context=context):
            result.append((vehicle.id, '%s (%s/%s)' % (vehicle.name,vehicle.make,vehicle.colour)))
        return result
        
    def create(self, cr, uid, ids, context=None):
        return     
        
    def _complete_name(self, cr, uid, ids, name, args, context=None):
        res = {}
        for m in self.browse(cr, uid, ids, context=context):
            res[m.id] = (m.parent_id and (m.parent_id.name + '/') or '') + m.name
        return res
        
    def create(self, cr, uid, vals, context=None):
        if not vals.get('location', False):
            vals.update({'location': self.pool.get('stock.location').create(cr,uid,{'name': vals['name']},context)})
        return super(fms_vehicles, self).create(cr, uid, vals, context=context)

    def _get_parent(self, cr, uid, context=None):
        return self.pool.get('ir.model.data').browse(cr, uid,self.pool.get('ir.model.data').search(cr, uid,[('name','=','account_analytic_account_parent'),('module','=','fms')]))[0].res_id

    def _get_user_messages(self, cr, uid, ids, field_name,arg, context=None):
        res = {}
        for id in ids:
            res[id] = []
            msg = self.pool.get('res.log').search(cr, uid, [('res_model', '=', 'fms.vehicles'),('res_id','=',id)], order='create_date desc',limit=100,context=context)
            if msg:
                res[id] = msg
        return res

    
    _columns = {
                'analytic_account_id': fields.many2one('account.analytic.account', 'Analytic Account', help="Link this device to an analytic account if you need financial management. It enables you to connect devices with budgets, planning, cost and revenue analysis.", ondelete="cascade", required=True),
                'year':fields.char('Year',size=4),
                'make':fields.char('Make',size=10),
                'model':fields.char('Model',size=15),
                'colour':fields.char('Colour',size=15),
                'serial':fields.char('productSerial #',size=50),
                'vehicle_type': fields.selection([
                        ('truck','Truck'),
                        ('bus','Bus'),
                        ('other','Other'),
                        ('car','Car')], 'Class', required=False,),
                'state': fields.selection([
                        ('draft','Draft'),
                        ('active','Active'),
                        ('outofservice','Out of Service'),
                        ('deprecated','Deprecated'),
                        ], 'State', required=True,),
                'ownership': fields.selection([
                        ('owned','Owned'),
                        ('private','Private (owned by employee)'),
                        ('leased','Leased'),
                        ('rented','Rented'),                       
                        ], 'Ownership', required=False), 
                        
                'bdate':fields.function(_amount_all,string='Recorded date',type = 'date',help="Date on which the mileage is recorded",multi="bdate"),
                'lodometer':fields.function(_amount_all,string='Last Odometer',type = 'integer',help="last recorded odometer",multi="lodometer"),
                'lmiles':fields.function(_amount_vehicle,string='Miles per year',type = 'integer',help="Miles per year",multi="lmiles"),
                'days':fields.function(_amount_vehicle,string='Days',type = 'float',help="Days that the vehicle has been on the road since purchase",multi="days"),
                'distance':fields.function(_amount_vehicle,string='Distance',type = 'float',help="Distance that the vehicle has been on the road since purchase",multi="distance"),
                
                'pdate':fields.date('Purchase Date',help="Date of Purchase of vehicle"),
                'pcost':fields.float('Purchase Value',digits=(16,2)),
                'ppartner':fields.many2one('res.partner','Purchased From'),
                'pinvoice':fields.many2one('account.invoice','Purchase Invoice',),
                'podometer':fields.integer('Odometer at Purchase'),
                'primarymeter':fields.selection([
                                                 ('odometer','Odometer'),
                                                 ('hourmeter','Hour Meter'),
                                                 ],'Primary Meter',required=False),
                'fueltype':fields.selection([
                                             ('95','95 (Euro)'),
                                             ('98','98 (Super)'),
                                             ('E85','E85 (Etanole)'),
                                             ('B','Biogas'),
                                             ('D','Diesel'),
                                             ('Electric','Electric'),
                                             ],'Fuel Used',required=False),
                'fueltankcap':fields.float('Fuel Tank Capacity'),
                'warrexp':fields.date('Date',help="Expiry date for warranty of product"),
                'warrexpmil':fields.integer('(or) Mileage',help="Expiry mileage for warranty of product"),
                'location':fields.many2one('stock.location','Stk Location',size=35,help="Select the stock location or create one for each vehicle(recommended) so that the spares, tyres etc are assossiated with the vehicle when issued",required=False),
                'expcut': fields.float('Expenditure Cut',help="Budgeted expenditure cut or alternative cost per month to calculate ROI"),
                'costpm':fields.function(_amount_vehicle,string='Cost per mile',type = 'float',help="Contract Cost per Month",multi="costpm"),
                'costpmon':fields.function(_amount_vehicle,string='Cost per month',type = 'float',help="Contract Cost per Month",multi="costpm"),
                'roi':fields.function(_amount_vehicle,string='ROI',type = 'float',help="Contract Return of Investment",multi="roi"),
                'log':fields.function(_get_user_messages,string='History',method=True,type = 'one2many',relation='res.log',),

                }
    _defaults={
               'vehicle_type':lambda *a:'car',
               'state':lambda *a:'draft',
               'ownership':lambda *a:'owned',
               'fueltype':lambda *a:'95',
               'primarymeter':lambda *a:'odometer',
               'company_id': lambda s, cr, uid, c: s.pool.get('res.company')._company_default_get(cr, uid, 'res.users', context=c),
               'parent_id': _get_parent,
               }

    def button_active(self, cr, uid, ids, context=None):
#        self.log(cr, uid, ids[0], "Here I come %s" % self.browse(cr,uid,ids[0]).name)
        return self.write(cr, uid, ids, {'state': 'active'})
    def button_deprecated(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'deprecated'})
    def button_outofservice(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'outofservice'})
    def button_draft(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {'state': 'draft'})
        
    def on_change_partner_id(self, cr, uid, id, partner_id, context={}):
        if not partner_id:
            return {'value': {'contact_id': False}}
        addr = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['invoice'])
        return {'value': {'contact_id': addr.get('invoice', False)}}

    def on_change_company(self, cr, uid, id, company_id):
        if not company_id:
            return {}
        currency = self.pool.get('res.company').read(cr, uid, [company_id], ['currency_id'])[0]['currency_id']
        return {'value': {'currency_id': currency}}


    def onchange_partner_id(self, cr, uid, ids, partner, context=None):
        partner_obj = self.pool.get('res.partner')
        if not partner:
            return {'value':{'contact_id': False}}
        address = partner_obj.address_get(cr, uid, [partner], ['contact'])
        return {'value':{'contact_id': address['contact']}}

    _sql_constraints = [
        ('uniq_name', 'unique (name)', 'The registration no (name) of the vehicle must be unique !')
    ]
    
fms_vehicles()

#class account_account(osv.osv):
    ##This module will 
    #_inherit = "account.account"
    #_columns = {
             #'type': fields.selection([
            #('receivable','Receivable'),
            #('payable','Payable'),
            #('view','View'),
            #('consolidation','Consolidation'),
            #('other','Others'),
            #('closed','Closed'),
            #('vehicle','Vehicle'),
        #], 'Internal Type', required=True,),
#}
#account_account()

class product_product(osv.osv):
    _name="product.product"
    _description="product fleet enhancements"
    _inherit="product.product"
    _columns={
              'spare_ok': fields.boolean('Is a vehicle spare', help="Determines if the product is a vehicle spare."),
              }
product_product()

class res_partner_address(osv.osv):
    _description ='Partner Addresses with fleet enhancements'
    _name = 'res.partner.address'
    _order = 'id'
    _inherit='res.partner.address'
    _columns = {
                'type': fields.selection( [ ('default','Default'),('invoice','Invoice'), ('delivery','Delivery'), ('contact','Contact'),('origin','Origin'),('destination','Destination'), ('other','Other') ],'Address Type', help="Used to select automatically the right address according to the context in sales and purchases documents."),
                }
res_partner_address()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
